package eu.DECIDEh2020.architect.plugin.builder;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.ide.IDE;

import com.fasterxml.jackson.core.JsonProcessingException;

import eu.DECIDEh2020.appManager.AppDescriptionFactory;
import eu.DECIDEh2020.appManager.AppDescriptionHelper;
import eu.DECIDEh2020.appManager.models.AppDescription;
import eu.DECIDEh2020.architect.plugin.Activator;
import eu.DECIDEh2020.architect.plugin.descriptorWizard.ConfigPage;
import eu.DECIDEh2020.architect.plugin.descriptorWizard.MicroservicePage;


public class BuilderWizard extends Wizard {
	private IProgressMonitor monitor;
	private AppDescription appdescription;
	private MicroservicePage microservicePage;
	IProject project;
	private ConfigPage configPage;
	
	public BuilderWizard(IProgressMonitor monitor, IProject project) {
		super();
		this.monitor= monitor;
		this.project= project;
		setWindowTitle("DECIDE project wizard");
		appdescription = AppDescriptionFactory.createAppDescription("DECIDE");
	}
	
	public void addPages() {
		super.addPages();

		configPage = new ConfigPage(appdescription);
		addPage(configPage);

		microservicePage = new MicroservicePage(appdescription);
		addPage(microservicePage);
	}
	
	@Override
	public boolean performFinish() {
		monitor.subTask("Adding DECIDE file");
		try {
			createFile(project, monitor);
		} catch (CoreException e) {
			Activator.getDefault().getLog().log(new Status(Status.ERROR, Activator.PLUGIN_ID, "Creating file.", e));
		}
		monitor.worked(1);
		return true;
	}

	@Override
	public IWizardPage getNextPage(IWizardPage currentPage) {
		if (currentPage == configPage) {
			microservicePage.setMicroservices();
			return microservicePage;
		}
		return null;
	}
	
	private void createFile(IProject project, IProgressMonitor monitor) throws CoreException {
		// create a sample file
		final String fileName = "DECIDE.json";
		monitor.beginTask("Creating " + fileName, 2);

		final IFile file = project.getFile(new Path(fileName));
		try {
			InputStream stream = openContentStream(appdescription);
			if (file.exists()) {
				file.setContents(stream, true, true, monitor);
			} else {
				file.create(stream, true, monitor);
			}
			stream.close();
		} catch (IOException e) {
		}
		monitor.worked(1);
		openFile(file,monitor);
	
	}
	private void openFile(IFile file, IProgressMonitor monitor) {
		monitor.setTaskName("Opening file for editing...");
		this.getShell().getDisplay().asyncExec(new Runnable() {
			public void run() {
				IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
				try {
					IDE.openEditor(page, file, true);
				} catch (PartInitException e) {
				}
			}
		});
		monitor.worked(1);
	}
	
	private InputStream openContentStream(AppDescription appdescription) throws CoreException {
		String contents = "";
		try {
			AppDescriptionHelper adh = AppDescriptionHelper.create(appdescription);
			contents = adh.getJson();
		} catch (JsonProcessingException e) {
			throw new CoreException(new Status(Status.ERROR, Activator.PLUGIN_ID, Status.OK, "Reading user generated JSON", e));
		}
		return new ByteArrayInputStream(contents.getBytes());
	}

}
