/*
* Copyright (c) 2017 Fraunhofer FOKUS.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the
* Eclipse Public License version 2.0 which accompanies
* this distribution, and is available at
* https://opensource.org/licenses/EPL-2.0
*
* Contributors:
*
* Torben Jastrow (Fraunhofer FOKUS) 
* 
* **Initially developed in the context of DECIDE EU project www.DECIDE-h2020.eu */

package eu.DECIDEh2020.architect.plugin.descriptorWizard;

import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Label;
import org.eclipse.wb.swt.SWTResourceManager;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;

import eu.DECIDEh2020.architect.plugin.layoutComponents.MicroserviceListComposite;
import eu.DECIDEh2020.architect.plugin.layoutComponents.PageCompleteListener;
import eu.DECIDEh2020.appManager.models.AppDescription;

//Second page of the wizard with project info and number of microservices
public class ConfigPage extends WizardPage {

    private Composite container;
    private AppDescription project;
    private MicroserviceListComposite microserviceList;
    private Text txtProjectdescription;

    /**
     * @wbp.parser.constructor
     */
    public ConfigPage(AppDescription project) {
	super("Project Configuration");

	this.project = project;

	setTitle("Project Configuration");
	setDescription("Configure project and set desired number of microservices");
    }

    @Override
    public void createControl(Composite parent) {
	container = new Composite(parent, SWT.NULL);
	setControl(container);
	container.setLayout(new GridLayout(1, false));
	addMicroserviceControls();
	setPageComplete(false);
    }

    private void addMicroserviceControls() {

	Composite compositeProjectDescription = new Composite(container, SWT.NONE);
	compositeProjectDescription.setLayout(new GridLayout(2, false));
	compositeProjectDescription.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));

	Label lblAppDescription = new Label(compositeProjectDescription, SWT.NONE);
	lblAppDescription.setLayoutData(new GridData(SWT.RIGHT, SWT.TOP, false, false, 1, 1));
	lblAppDescription.setText("App Description:");

	txtProjectdescription = new Text(compositeProjectDescription, SWT.BORDER | SWT.MULTI);
	txtProjectdescription.addKeyListener(new KeyAdapter() {
	    @Override
	    public void keyReleased(KeyEvent e) {
		project.setDescription(txtProjectdescription.getText());
	    }
	});
	txtProjectdescription.setText("ProjectDescription");
	GridData gd_txtProjectdescription = new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1);
	gd_txtProjectdescription.heightHint = 52;
	txtProjectdescription.setLayoutData(gd_txtProjectdescription);
	new Label(compositeProjectDescription, SWT.NONE);
	new Label(compositeProjectDescription, SWT.NONE);

	Group grpMicroservices = new Group(container, SWT.NONE);

	grpMicroservices.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 1, 1));
	grpMicroservices.setText("Microservices");
	grpMicroservices.setLayout(new GridLayout(1, false));

	microserviceList = new MicroserviceListComposite(grpMicroservices, SWT.NONE, project, true, SWTResourceManager.getColor(SWT.COLOR_WIDGET_BACKGROUND));
	microserviceList.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));

	microserviceList.addPageCompleteListener(new PageCompleteListener() {
	    @Override
	    public void pageCompleted(Boolean complete) {
		setPageComplete(complete);
	    }
	});

    }

    public void setVisible(boolean visible) {
	super.setVisible(visible);
	if (visible) {
	    microserviceList.refresh();
	}
    }

}
