package eu.DECIDEh2020.architect.plugin.layoutComponents;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.wb.swt.SWTResourceManager;

import eu.DECIDEh2020.appManager.AppDescriptionHelper;
import eu.DECIDEh2020.appManager.models.AppDescription;
import eu.DECIDEh2020.appManager.models.AvailabilityNfr;
import eu.DECIDEh2020.appManager.models.Nfr;
import eu.DECIDEh2020.appManager.models.Pattern;
import eu.DECIDEh2020.appManager.models.PerformanceNfr;
import eu.DECIDEh2020.architect.plugin.utils.CloudPatternHelper;

import org.eclipse.jface.viewers.ListViewer;
import org.eclipse.swt.widgets.Button;
import org.eclipse.jface.viewers.EditingSupport;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.ComboBoxCellEditor;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerFilter;
import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.swt.graphics.Image;
import org.eclipse.jface.viewers.LabelProvider;

//composite for the NFR Editor
public class NfrPageComposite extends Composite {
	private static class PatternLabelProvider extends LabelProvider {
		public Image getImage(Object element) {
			return null;
		}

		public String getText(Object element) {
			return ((Pattern) element).getTitle();
		}
	}

	private static class PatternContentProvider implements IStructuredContentProvider {
		@SuppressWarnings("unchecked")
		public Object[] getElements(Object inputElement) {

			return ((java.util.List<Pattern>) inputElement).toArray();

		}

		public void dispose() {
		}

		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
		}
	}

	public class NfrFilter extends ViewerFilter {

		private String type;

		public void setType(String type) {
			// ensure that the value can be used for matching
			this.type = type;
		}

		@Override
		public boolean select(Viewer viewer, Object parentElement, Object element) {
			if (type == null || type.length() == 0) {
				return true;
			}
			Nfr p = (Nfr) element;
			if (p.getType().equals(type)) {
				return true;
			}

			return false;
		}
	}

	public class NfrNameLabelProvider extends ColumnLabelProvider {
		public Image getImage(Object element) {
			// TODO Auto-generated method stub
			return null;
		}

		public String getText(Object element) {

			if (((Nfr) element).getTags().size() > 0) {
				return ((Nfr) element).getTags().get(0);
			} else {
				return null;
			}
		}
	}

	public class NfrUnitLabelProvider extends ColumnLabelProvider {
		public Image getImage(Object element) {
			return null;
		}

		public String getText(Object element) {
			if (((Nfr) element).getAdditionalProperties().containsKey("unit")) {
				return ((Nfr) element).getAdditionalProperties().get("unit").asText();
			} else if (PerformanceNfr.class.isInstance(element)) {
				return ((PerformanceNfr) element).getUnit();
			} else if (AvailabilityNfr.class.isInstance(element)) {
				return ((AvailabilityNfr) element).getUnit();
			} else {
				return null;
			}

		}
	}

	public class TagEditingSupport extends EditingSupport {
		private final TableViewer viewer;
		private final ComboBoxCellEditor editor;

		public TagEditingSupport(TableViewer viewer) {
			super(viewer);
			this.viewer = viewer;
			AppDescriptionHelper adh = AppDescriptionHelper.create(appDescription);
			this.editor = new ComboBoxCellEditor(viewer.getTable(),
					adh.getTags().toArray(new String[adh.getTags().size()]));
		}

		@Override
		protected CellEditor getCellEditor(Object element) {
			AppDescriptionHelper adh = AppDescriptionHelper.create(appDescription);
			editor.setItems(adh.getTags().toArray(new String[adh.getTags().size()]));
			return editor;
		}

		@Override
		protected boolean canEdit(Object element) {
			return true;
		}

		@Override
		protected Object getValue(Object element) {
			Nfr n = (Nfr) element;
			String[] items = ((ComboBoxCellEditor) editor).getItems();
			int idx = 0;
			for (int i = 0; i < items.length; i++) {
				if (items[i].equals((n.getTags().size() > 0 ? n.getTags().get(0) : ""))) {
					idx = i;
					break;
				}
			}

			return idx;
		}

		@Override
		protected void setValue(Object element, Object value) {
			String[] items = ((ComboBoxCellEditor) editor).getItems();
			String selected = items[(Integer) value];

			((Nfr) element).setTags(Collections.singletonList(selected));

			viewer.update(element, null);
		}

	}

	private Color background;
	private AppDescription appDescription;
	private AppDescriptionHelper helper;
	private Table table_location;
	private TableViewer tableViewer_performance;
	private TableViewer tableViewer_availability;
	private List<Nfr> nfrs;
	private Table table;
	private ListViewer listViewerPatterns;
	private org.eclipse.swt.widgets.List listPatterns;

	public NfrPageComposite(Composite parent, AppDescription appDescription) {
		super(parent, SWT.NONE);

		this.appDescription = appDescription;
		this.nfrs = appDescription.getNfrs();

		helper = AppDescriptionHelper.create(appDescription);

		background = SWTResourceManager.getColor(SWT.COLOR_WHITE);
		setBackground(background);
		createContent();
		fillContent();
		requestLayout();
	}

	private void createContent() {
		Control[] children = this.getChildren();
		for (int i = 0; i < children.length; i++) {
			children[i].dispose();
		}
		setLayout(new GridLayout(2, false));

		Composite composite_8 = new Composite(this, SWT.NONE);
		composite_8.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		composite_8.setLayout(new GridLayout(1, false));

		CLabel lblOverview = new CLabel(composite_8, SWT.NONE);
		lblOverview.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		lblOverview.setSize(44, 27);
		lblOverview.setFont(SWTResourceManager.getFont("Segoe UI", 12, SWT.BOLD));
		lblOverview.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		lblOverview.setText("NFRs");

		Label label = new Label(composite_8, SWT.SEPARATOR | SWT.HORIZONTAL);
		label.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		label.setSize(236, 2);

		CLabel lblScalability = new CLabel(composite_8, SWT.NONE);
		lblScalability.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		lblScalability.setSize(61, 15);
		lblScalability.setTopMargin(0);
		lblScalability.setText("Scalability");
		lblScalability.setFont(SWTResourceManager.getFont("Segoe UI", 9, SWT.BOLD));
		lblScalability.setBottomMargin(0);
		lblScalability.setBackground(SWTResourceManager.getColor(SWT.COLOR_WIDGET_BACKGROUND));

		Composite composite = new Composite(composite_8, SWT.NONE);
		composite.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		composite.setSize(236, 131);
		composite.setLayout(new GridLayout(2, false));

		TableViewer tableViewer_3 = new TableViewer(composite, SWT.BORDER | SWT.FULL_SELECTION);
		table = tableViewer_3.getTable();
		table.setLinesVisible(true);
		table.setHeaderVisible(true);
		GridData gd_table = new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1);
		gd_table.heightHint = 100;
		table.setLayoutData(gd_table);

		TableViewerColumn tableViewerColumn_7 = new TableViewerColumn(tableViewer_3, SWT.NONE);
		TableColumn tblclmnNameservicegroup = tableViewerColumn_7.getColumn();
		tblclmnNameservicegroup.setWidth(100);
		tblclmnNameservicegroup.setText("Name (µService/Group)");

		TableViewerColumn tableViewerColumn_8 = new TableViewerColumn(tableViewer_3, SWT.NONE);
		TableColumn tblclmnScaleBy = tableViewerColumn_8.getColumn();
		tblclmnScaleBy.setWidth(100);
		tblclmnScaleBy.setText("Scale by");

		Composite composite_7 = new Composite(composite, SWT.NONE);
		composite_7.setLayoutData(new GridData(SWT.LEFT, SWT.BOTTOM, false, false, 1, 1));
		composite_7.setLayout(new FillLayout(SWT.VERTICAL));

		Button btnAdd_1 = new Button(composite_7, SWT.NONE);
		btnAdd_1.setEnabled(false);
		btnAdd_1.setText("Add");

		Button btnRemove_1 = new Button(composite_7, SWT.NONE);
		btnRemove_1.setText("Remove");

		CLabel lblPerformance = new CLabel(composite_8, SWT.NONE);
		lblPerformance.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		lblPerformance.setSize(301, 15);
		lblPerformance.setTopMargin(0);
		lblPerformance.setText("Performance");
		lblPerformance.setFont(SWTResourceManager.getFont("Segoe UI", 9, SWT.BOLD));
		lblPerformance.setBottomMargin(0);
		lblPerformance.setBackground(SWTResourceManager.getColor(SWT.COLOR_WIDGET_BACKGROUND));

		Composite composite_1 = new Composite(composite_8, SWT.NONE);
		composite_1.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		composite_1.setSize(301, 131);
		composite_1.setLayout(new GridLayout(2, false));

		tableViewer_performance = new TableViewer(composite_1, SWT.BORDER | SWT.FULL_SELECTION);
		Table table_performance = tableViewer_performance.getTable();
		table_performance.setLinesVisible(true);
		table_performance.setHeaderVisible(true);
		GridData gd_table_performance = new GridData(SWT.FILL, SWT.FILL, true, false, 1, 1);
		gd_table_performance.heightHint = 100;
		table_performance.setLayoutData(gd_table_performance);
		NfrFilter perfFilter = new NfrFilter();
		perfFilter.setType("Performance");
		tableViewer_performance.setFilters(perfFilter);

		TableViewerColumn tableViewerColumn_2 = new TableViewerColumn(tableViewer_performance, SWT.NONE);
		tableViewerColumn_2.setEditingSupport(new TagEditingSupport(tableViewer_performance));
		tableViewerColumn_2.setLabelProvider(new NfrNameLabelProvider());

		TableColumn tblclmnNewColumn = tableViewerColumn_2.getColumn();
		tblclmnNewColumn.setWidth(100);
		tblclmnNewColumn.setText("Name (µService/Group)");

		TableViewerColumn tableViewerColumn_3 = new TableViewerColumn(tableViewer_performance, SWT.NONE);
		tableViewerColumn_3.setEditingSupport(new EditingSupport(tableViewer_performance) {
			protected boolean canEdit(Object element) {
				// TODO Auto-generated method stub
				return true;
			}

			protected CellEditor getCellEditor(Object element) {
				// TODO Auto-generated method stub
				return new TextCellEditor(tableViewer_performance.getTable());
			}

			protected Object getValue(Object element) {
				// TODO Auto-generated method stub
				return String.valueOf(((PerformanceNfr) element).getValue());
			}

			protected void setValue(Object element, Object value) {
				PerformanceNfr n = (PerformanceNfr) element;
				Double d = Double.valueOf((String) value);
				n.setValue(d);

				tableViewer_performance.update(element, null);
			}
		});
		tableViewerColumn_3.setLabelProvider(new ColumnLabelProvider() {
			public Image getImage(Object element) {
				// TODO Auto-generated method stub
				return null;
			}

			public String getText(Object element) {
				
				return String.valueOf(((PerformanceNfr) element).getValue());
				
			}
		});
		TableColumn tblclmnResponseTime = tableViewerColumn_3.getColumn();
		tblclmnResponseTime.setWidth(100);
		tblclmnResponseTime.setText("Response Time");

		TableViewerColumn tableViewerColumn_5 = new TableViewerColumn(tableViewer_performance, SWT.NONE);
		tableViewerColumn_5.setLabelProvider(new NfrUnitLabelProvider());
		TableColumn tblclmnUnit = tableViewerColumn_5.getColumn();
		tblclmnUnit.setWidth(100);
		tblclmnUnit.setText("Unit");

		tableViewer_performance.setContentProvider(ArrayContentProvider.getInstance());
		tableViewer_performance.setInput(nfrs);

		Composite composite_4 = new Composite(composite_1, SWT.NONE);
		composite_4.setLayoutData(new GridData(SWT.LEFT, SWT.BOTTOM, false, true, 1, 1));
		composite_4.setLayout(new FillLayout(SWT.VERTICAL));

		Button btnAddPerf = new Button(composite_4, SWT.NONE);

		btnAddPerf.setText("Add");

		Button btnRemovePerf = new Button(composite_4, SWT.NONE);

		btnRemovePerf.setText("Remove");

		CLabel lblAvailability = new CLabel(composite_8, SWT.NONE);
		lblAvailability.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		lblAvailability.setSize(301, 15);
		lblAvailability.setTopMargin(0);
		lblAvailability.setText("Availability");
		lblAvailability.setFont(SWTResourceManager.getFont("Segoe UI", 9, SWT.BOLD));
		lblAvailability.setBottomMargin(0);
		lblAvailability.setBackground(SWTResourceManager.getColor(SWT.COLOR_WIDGET_BACKGROUND));

		Composite composite_3 = new Composite(composite_8, SWT.NONE);
		composite_3.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		composite_3.setSize(301, 60);
		composite_3.setLayout(new GridLayout(2, false));

		tableViewer_availability = new TableViewer(composite_3, SWT.BORDER | SWT.FULL_SELECTION);
		Table table_availability = tableViewer_availability.getTable();
		table_availability.setLinesVisible(true);
		table_availability.setHeaderVisible(true);
		GridData gd_table_availability = new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1);
		gd_table_availability.heightHint = 100;
		table_availability.setLayoutData(gd_table_availability);
		NfrFilter availFilter = new NfrFilter();
		availFilter.setType("Availability");
		tableViewer_availability.setFilters(availFilter);

		TableViewerColumn tableViewerColumn_4 = new TableViewerColumn(tableViewer_availability, SWT.NONE);
		tableViewerColumn_4.setEditingSupport(new TagEditingSupport(tableViewer_availability));
		tableViewerColumn_4.setLabelProvider(new NfrNameLabelProvider());
		TableColumn tblclmnName_1 = tableViewerColumn_4.getColumn();
		tblclmnName_1.setWidth(100);
		tblclmnName_1.setText("Name (µService/Group)");

		TableViewerColumn tableViewerColumn_6 = new TableViewerColumn(tableViewer_availability, SWT.NONE);
		tableViewerColumn_6.setEditingSupport(new EditingSupport(tableViewer_availability) {
			protected boolean canEdit(Object element) {
				// TODO Auto-generated method stub
				return true;
			}

			protected CellEditor getCellEditor(Object element) {
				// TODO Auto-generated method stub
				return new TextCellEditor(tableViewer_availability.getTable());
			}

			protected Object getValue(Object element) {
				// TODO Auto-generated method stub
				return String.valueOf(((AvailabilityNfr) element).getValue());
			}

			protected void setValue(Object element, Object value) {
				AvailabilityNfr n = (AvailabilityNfr) element;
				Double d = Double.valueOf((String) value);
				n.setValue(d);
				tableViewer_availability.update(element, null);
			}
		});
		tableViewerColumn_6.setLabelProvider(new ColumnLabelProvider() {
			public Image getImage(Object element) {
				// TODO Auto-generated method stub
				return null;
			}

			public String getText(Object element) {
				return String.valueOf(((AvailabilityNfr) element).getValue());
			}
		});
		TableColumn tblclmnQuant = tableViewerColumn_6.getColumn();
		tblclmnQuant.setWidth(100);
		tblclmnQuant.setText("Quantitative Availability");

		TableViewerColumn tableViewerColumn_9 = new TableViewerColumn(tableViewer_availability, SWT.NONE);
		tableViewerColumn_9.setLabelProvider(new NfrUnitLabelProvider());
		TableColumn tblclmnUnit_1 = tableViewerColumn_9.getColumn();
		tblclmnUnit_1.setWidth(100);
		tblclmnUnit_1.setText("Unit");

		tableViewer_availability.setContentProvider(ArrayContentProvider.getInstance());
		tableViewer_availability.setInput(nfrs);

		Composite composite_5 = new Composite(composite_3, SWT.NONE);
		composite_5.setLayoutData(new GridData(SWT.LEFT, SWT.BOTTOM, false, false, 1, 1));
		composite_5.setLayout(new FillLayout(SWT.VERTICAL));

		Button btnAddAvail = new Button(composite_5, SWT.NONE);

		btnAddAvail.setText("Add");

		Button btnRemoveAvail = new Button(composite_5, SWT.NONE);

		btnRemoveAvail.setText("Remove");

		CLabel lblLocation = new CLabel(composite_8, SWT.NONE);
		lblLocation.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		lblLocation.setSize(301, 15);
		lblLocation.setTopMargin(0);
		lblLocation.setText("Location");
		lblLocation.setFont(SWTResourceManager.getFont("Segoe UI", 9, SWT.BOLD));
		lblLocation.setBottomMargin(0);
		lblLocation.setBackground(SWTResourceManager.getColor(SWT.COLOR_WIDGET_BACKGROUND));

		Composite composite_2 = new Composite(composite_8, SWT.NONE);
		composite_2.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		composite_2.setSize(301, 45);
		composite_2.setLayout(new GridLayout(2, false));

		TableViewer tableViewer_location = new TableViewer(composite_2, SWT.BORDER | SWT.FULL_SELECTION);
		table_location = tableViewer_location.getTable();
		GridData gd_table_location = new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1);
		gd_table_location.heightHint = 100;
		table_location.setLayoutData(gd_table_location);
		table_location.setHeaderVisible(true);
		table_location.setLinesVisible(true);

		TableViewerColumn tableViewerColumn = new TableViewerColumn(tableViewer_location, SWT.NONE);
		TableColumn tblclmnName = tableViewerColumn.getColumn();
		tblclmnName.setText("Name");

		TableViewerColumn tableViewerColumn_1 = new TableViewerColumn(tableViewer_location, SWT.NONE);
		TableColumn tblclmnRegion = tableViewerColumn_1.getColumn();
		tblclmnRegion.setText("Region");

		Composite composite_9 = new Composite(composite_2, SWT.NONE);
		composite_9.setLayout(new FillLayout(SWT.VERTICAL));
		composite_9.setLayoutData(new GridData(SWT.LEFT, SWT.BOTTOM, false, false, 1, 1));

		Button btnAddLoc = new Button(composite_9, SWT.NONE);
		btnAddLoc.setEnabled(false);
		btnAddLoc.setBounds(0, 0, 75, 25);
		btnAddLoc.setText("Add");

		Button btnRemoveLoc = new Button(composite_9, SWT.NONE);
		btnRemoveLoc.setText("Remove");

		Composite composite_6 = new Composite(this, SWT.NONE);
		composite_6.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		composite_6.setLayout(new GridLayout(1, false));

		listViewerPatterns = new ListViewer(composite_6, SWT.BORDER | SWT.V_SCROLL);
		listPatterns = listViewerPatterns.getList();
		listPatterns.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		listViewerPatterns.setLabelProvider(new PatternLabelProvider());
		listViewerPatterns.setContentProvider(new PatternContentProvider());
		listViewerPatterns.setInput(CloudPatternHelper.inferPatterns(nfrs));
		listViewerPatterns.refresh();

		btnAddAvail.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {

				AvailabilityNfr n = new AvailabilityNfr();
				n.setType("Availability");
				List<String> l = new ArrayList<String>();
				l.add("Application");
				n.setTags(l);

				n.setUnit("percentage");
				n.setValue(0.00);
				n.setAbstractValue("Medium");
				addnfr(n);
				tableViewer_availability.refresh();
				tableViewer_availability.getTable().update();
				listViewerPatterns.setInput(CloudPatternHelper.inferPatterns(nfrs));
				listViewerPatterns.refresh();
				listPatterns.update();
			}
		});

		btnRemoveAvail.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				int idx = table_availability.getSelectionIndex();
				Nfr selected = (Nfr) tableViewer_availability.getElementAt(idx);
				removenfr(selected);
				tableViewer_availability.refresh();
				listViewerPatterns.setInput(CloudPatternHelper.inferPatterns(nfrs));
				listViewerPatterns.refresh();
			}
		});
		btnAddPerf.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				PerformanceNfr n = new PerformanceNfr();
				n.setType("Performance");
				List<String> l = new ArrayList<String>();
				l.add("Application");
				n.setTags(l);
				n.setAbstractValue("Medium");
				n.setValue(0.00);
				n.setUnit("milliseconds");
				addnfr(n);
				tableViewer_performance.refresh();
				listViewerPatterns.setInput(CloudPatternHelper.inferPatterns(nfrs));
				listViewerPatterns.refresh();
			}
		});

		btnRemovePerf.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				int idx = table_performance.getSelectionIndex();

				Nfr selected = (Nfr) tableViewer_performance.getElementAt(idx);
				removenfr(selected);
				tableViewer_performance.refresh();
				listViewerPatterns.setInput(CloudPatternHelper.inferPatterns(nfrs));
				listViewerPatterns.refresh();
			}
		});

		requestLayout();

	}

	private void addnfr(Nfr n) {
		this.nfrs.add(n);
		this.appDescription.setNfrs(nfrs);

		fillContent();
	}

	private void removenfr(Nfr n) {
		this.nfrs.remove(n);
		this.appDescription.setNfrs(nfrs);

		fillContent();
	}

	public void refreshContent(AppDescription appDescription) {
		this.appDescription = appDescription;
		this.nfrs = appDescription.getNfrs();
		if (nfrs == null) {
			nfrs = new ArrayList<Nfr>();
		}
		fillContent();
	}

	private void fillContent() {

		tableViewer_availability.setInput(nfrs);
		tableViewer_performance.setInput(nfrs);

	}
}
