/*
* Copyright (c) 2017 Fraunhofer FOKUS.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the
* Eclipse Public License version 2.0 which accompanies
* this distribution, and is available at
* https://opensource.org/licenses/EPL-2.0
*
* Contributors:
*
* Torben Jastrow (Fraunhofer FOKUS) 
* 
* **Initially developed in the context of DECIDE EU project www.DECIDE-h2020.eu */
package eu.DECIDEh2020.architect.plugin.layoutComponents;

import java.util.ArrayList;

import org.eclipse.jface.viewers.ListViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.custom.CTabFolder2Listener;
import org.eclipse.swt.custom.CTabFolderEvent;
import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowData;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Widget;
import org.eclipse.wb.swt.ResourceManager;
import org.eclipse.wb.swt.SWTResourceManager;

import eu.DECIDEh2020.appManager.models.Nfr;

//first version of a NFR Composite, not used
public class NfrSelector extends Composite {
	private  ArrayList<nfr> nfrList = new ArrayList<nfr>();
	private Text txtCost;
	private Text text;
	
	private class nfr{
		private CTabItem tabItem;
		private Button button;
		private CTabFolder parentFolder;
		private Composite buttonParent;
		private Composite content;
		private String name;
		private String toolTipText;
		private Nfr nfrObject;
		
		public nfr(String name, CTabFolder itemParent, Composite itemContent, Composite buttonParent, String toolTipText) {
			this.name= name;
			this.parentFolder=itemParent;
			this.content = itemContent;
			this.buttonParent = buttonParent;
			this.toolTipText = toolTipText;
			
			addButton();
			addListener();
		}
		
		private void createObject() {
			//nfrObject= new Nfr();
			
		}
		private String getName() {
			return name;
		}
		private void addItem() {
			tabItem = new CTabItem(parentFolder, SWT.NONE);
			tabItem.setShowClose(true);
			tabItem.setToolTipText(toolTipText);
			tabItem.setText(name);
			tabItem.setControl(content);
			parentFolder.setSelection(tabItem);
		}
		
		private void addButton() {
			button = new Button(buttonParent, SWT.NONE);
			button.setImage(ResourceManager.getPluginImage("eu.DECIDEh2020.architect.plugin", "icons/plus_small.png"));
			button.setText(name);
			
			buttonParent.redraw();
			buttonParent.layout();
			buttonParent.getParent().layout();
			//System.out.println("button added");
		}
		
		public void closeItem() {
			//System.out.println("dispose");
			tabItem.dispose();
			tabItem = null;
			addButton();
			addListener();
		}
		
		private void addListener() {
			button.addSelectionListener(new SelectionAdapter() {
				@Override
				public void widgetSelected(SelectionEvent e) {
					button.dispose();
					
					button= null;
					addItem();
					buttonParent.redraw();
					buttonParent.layout();
					
				}
			});
			
			
		}
		
	}
	
	
	
	public NfrSelector(Composite parent, int style) {
		super(parent, style);
		// TODO Auto-generated constructor stub
		createGUI();
	}
	
	void createGUI(){
		setLayout(new GridLayout(1, false));
		
		
		
		CTabFolder tabFolder = new CTabFolder(this, SWT.BORDER);
		
		tabFolder.setUnselectedCloseVisible(false);
		tabFolder.setForeground(SWTResourceManager.getColor(SWT.COLOR_WIDGET_FOREGROUND));
		tabFolder.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 1, 1));
		
		tabFolder.addCTabFolder2Listener(new CTabFolder2Listener() {

		      public void close(CTabFolderEvent arg0) {
		        nfr n = findNfrInList(arg0.item);
		        	
		        if (n!=null) {
		        	n.closeItem();
		        }
		      }

		      public void minimize(CTabFolderEvent arg0) {
		        // TODO Auto-generated method stub

		      }

		      public void maximize(CTabFolderEvent arg0) {
		        // TODO Auto-generated method stub

		      }

		      public void restore(CTabFolderEvent arg0) {
		        // TODO Auto-generated method stub

		      }

			@Override
			public void showList(CTabFolderEvent event) {
				// TODO Auto-generated method stub
				
			}

		    });
		
		
		Composite compositeAvailability = new Composite(tabFolder, SWT.NONE);
		compositeAvailability.setLayout(new GridLayout(1, false));
		
		Label lblSelectTheTargeted = new Label(compositeAvailability, SWT.WRAP);
		lblSelectTheTargeted.setText("Select the targeted availability SLO range for the App as a whole:");
		
		Composite composite = new Composite(compositeAvailability, SWT.NONE);
		RowLayout rl_composite = new RowLayout(SWT.VERTICAL);
		rl_composite.marginLeft = 15;
		rl_composite.pack = false;
		composite.setLayout(rl_composite);
		
		Button btnA = new Button(composite, SWT.RADIO);
		btnA.setText("A > 99%");
		
		Button btnA_1 = new Button(composite, SWT.RADIO);
		btnA_1.setText("99% > A > 98%");
		
		Button btnA_2 = new Button(composite, SWT.RADIO);
		btnA_2.setText("98% > A");
		
		
		
		Composite compositeCost = new Composite(tabFolder, SWT.NONE);
		
		compositeCost.setLayout(new GridLayout(1, false));
		
		Label lblSelectTheTargeted_1 = new Label(compositeCost, SWT.WRAP);
		lblSelectTheTargeted_1.setText("Select the targeted cost SLO for the App as a whole:");
		
		Composite composite_2 = new Composite(compositeCost, SWT.NONE);
		composite_2.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1));
		FillLayout fl_composite_2 = new FillLayout(SWT.HORIZONTAL);
		fl_composite_2.spacing = 15;
		composite_2.setLayout(fl_composite_2);
		
		Label lblMaximumCost = new Label(composite_2, SWT.WRAP);
		lblMaximumCost.setAlignment(SWT.LEFT);
		lblMaximumCost.setText("Maximum cost:");
		
		txtCost = new Text(composite_2, SWT.BORDER);
		
	
		
		Composite compositePerformance = new Composite(tabFolder, SWT.NONE);
		compositePerformance.setLayout(new GridLayout(1, false));
		
		Label lblSelectTheTargeted_2 = new Label(compositePerformance, SWT.NONE);
		lblSelectTheTargeted_2.setText("Select the targeted performance SLO for the App as a whole:");
		
		Composite composite_3 = new Composite(compositePerformance, SWT.NONE);
		composite_3.setLayoutData(new GridData(SWT.FILL, SWT.BOTTOM, false, false, 1, 1));
		composite_3.setLayout(new FillLayout(SWT.HORIZONTAL));
		
		Label lblMaximumResponseTime = new Label(composite_3, SWT.NONE);
		lblMaximumResponseTime.setText("Maximum response time:");
		
		text = new Text(composite_3, SWT.BORDER);
		
		
		
		Composite compositeLocation = new Composite(tabFolder, SWT.NONE);
		
		
		compositeLocation.setLayout(new GridLayout(1, false));
		
		Label lblSelectTheTargeted_3 = new Label(compositeLocation, SWT.NONE);
		lblSelectTheTargeted_3.setText("Select the targeted location SLO  for the App as a whole:");
		
		Combo combo = new Combo(compositeLocation, SWT.NONE);
		combo.setItems(new String[] {"Europe", "USA", "Other"});
		combo.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1));
		
		
		
		Composite compositeSecurity = new Composite(tabFolder, SWT.NONE);
		compositeSecurity.setLayout(new GridLayout(1, false));
		
		Label lblSelectTheTargeted_4 = new Label(compositeSecurity, SWT.NONE);
		lblSelectTheTargeted_4.setText("Select the targeted Security/legal description SLO  for the App as a whole:");
		
		Composite composite_4 = new Composite(compositeSecurity, SWT.NONE);
		RowLayout rl_composite_4 = new RowLayout(SWT.HORIZONTAL);
		rl_composite_4.spacing = 15;
		rl_composite_4.justify = true;
		rl_composite_4.fill = true;
		rl_composite_4.marginWidth = 3;
		composite_4.setLayout(rl_composite_4);
		
		Label lblLegislationsecurityNeeds = new Label(composite_4, SWT.NONE);
		lblLegislationsecurityNeeds.setText("Legislation/security needs");
		
		ListViewer listViewer = new ListViewer(composite_4, SWT.BORDER | SWT.V_SCROLL);
		List list = listViewer.getList();
		list.setItems(new String[] {"GDPR", "Code of Conduct", "Encryption"});
		list.setLayoutData(new RowData(253, SWT.DEFAULT));
		
		Composite composite_1 = new Composite(this, SWT.NONE);
		composite_1.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, true, false, 1, 1));
		composite_1.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 1, 1));
		composite_1.setLayout(new GridLayout(2, false));
		
		Label lblClickToAdd = new Label(composite_1, SWT.CENTER);
		lblClickToAdd.setText("Click to add nfr to Project:");
		
		Composite composite_nfr_buttons = new Composite(composite_1, SWT.NONE);
		composite_nfr_buttons.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, true, false, 1, 1));
		RowLayout rl_composite_nfr_buttons = new RowLayout(SWT.HORIZONTAL);
		rl_composite_nfr_buttons.center = true;
		rl_composite_nfr_buttons.spacing = 5;
		composite_nfr_buttons.setLayout(rl_composite_nfr_buttons);
		
		nfr n = new nfr("Availability", tabFolder,compositeAvailability,composite_nfr_buttons, "DECIDE\u2019s availability description");
		
		nfrList.add(n);
		n = new nfr("Cost", tabFolder,compositeCost,composite_nfr_buttons, "DECIDE\u2019s cost description");
		nfrList.add(n);
		n = new nfr("Performance", tabFolder,compositePerformance,composite_nfr_buttons, "DECIDE\u2019s performance description");
		nfrList.add(n);
		n = new nfr("Location", tabFolder,compositeLocation,composite_nfr_buttons, "DECIDE\u2019s location description");
		nfrList.add(n);
		n = new nfr("Security(legal)", tabFolder,compositeSecurity,composite_nfr_buttons, "DECIDE\u2019s Security/legal description");
		nfrList.add(n);
		
		
	}
	
	private nfr findNfrInList(Widget tabItem) {
		for (nfr nfr : nfrList) {
			if(tabItem.equals(nfr.tabItem)) {
				System.out.println("found nfr");
				return nfr;
			}
		}
		System.out.println("found  nothing");
		return null;
	}
}
