/*
* Copyright (c) 2017 Fraunhofer FOKUS.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the
* Eclipse Public License version 2.0 which accompanies
* this distribution, and is available at
* https://opensource.org/licenses/EPL-2.0
*
* Contributors:
*
* Torben Jastrow (Fraunhofer FOKUS) 
* 
* **Initially developed in the context of DECIDE EU project www.DECIDE-h2020.eu */
package eu.DECIDEh2020.architect.plugin.layoutComponents;

import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.List;
import org.eclipse.wb.swt.SWTResourceManager;

import eu.DECIDEh2020.appManager.AppDescriptionHelper;
import eu.DECIDEh2020.appManager.models.AppDescription;
import eu.DECIDEh2020.appManager.models.Microservice;
import eu.DECIDEh2020.architect.cloudpatterns.Patterns;

import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;
import java.util.stream.Collectors;

import org.apache.jena.rdf.model.Resource;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.ListViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Button;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.TraverseListener;
import org.eclipse.swt.events.TraverseEvent;
//composite for the list of tags a microservice can have
public class TagListViewer extends Composite {
	private static class TagLabelProvider extends LabelProvider {
		public Image getImage(Object element) {
			return null;
		}

		public String getText(Object element) {
			return (String) element;
		}
	}

	private static class TagContentProvider implements IStructuredContentProvider {
		@SuppressWarnings("unchecked")
		public Object[] getElements(Object inputElement) {
			return ((Set<String>) inputElement).toArray();

		}

		public void dispose() {
		}

		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
			viewer.refresh();
		}
	}

	private List list;
	private ListViewer listViewer;
	private AppDescription appDescription;
	private AppDescriptionHelper appDescriptionHelper;
	private Set<String> allTags;
	private Set<String> microserviceTags;
	private Color background;

	/**
	 * Create the composite.
	 * 
	 * @param parent
	 * @param style
	 * @param microservice
	 */
	public TagListViewer(Composite parent, int style, Microservice microservice, AppDescription appDescription,Color background) {
		super(parent, SWT.NONE);
		this.background=background;
		this.setBackground(background);
		this.appDescription = appDescription;
		appDescriptionHelper = AppDescriptionHelper.create(appDescription);
		if (appDescription != null) {
			allTags = appDescriptionHelper.getTags();
		} else {
			allTags = new HashSet<String>();
		}
		Patterns patterns = Patterns.createDefaultPatterns();
		java.util.List<Resource> nfrs = patterns.listNFRs();
		setLayout(new GridLayout(1, false));
		
		listViewer = new ListViewer(this, SWT.BORDER);
		list = listViewer.getList();
		GridData gd_list = new GridData(SWT.FILL, SWT.CENTER, true, true, 1, 1);
		gd_list.minimumHeight = 75;
		list.setLayoutData(gd_list);

		Composite composite = new Composite(this, SWT.NONE);
		composite.setBackground(background);
		composite.setLayout(new GridLayout(5, false));
		composite.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));

		Label lblAddTag = new Label(composite, SWT.NONE);
		lblAddTag.setBackground(background);
		lblAddTag.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
		lblAddTag.setText("Add Tag:");

		ComboViewer comboViewer = new ComboViewer(composite, SWT.NONE);
		Combo combo = comboViewer.getCombo();
		
		
		combo.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));

		comboViewer.setContentProvider(new ArrayContentProvider());
		comboViewer.setInput(allTags);

		Button btnAdd = new Button(composite, SWT.NONE);

		GridData gd_btnAdd = new GridData(SWT.LEFT, SWT.CENTER, false, false, 1, 1);
		gd_btnAdd.widthHint = 50;
		btnAdd.setLayoutData(gd_btnAdd);
		btnAdd.setText("Add");

		Composite composite_1 = new Composite(composite, SWT.NONE);
		composite_1.setBackground(background);
		composite_1.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, true, false, 1, 1));
		composite_1.setLayout(new GridLayout(1, false));

		Button btnRemove = new Button(composite, SWT.NONE);

		GridData gd_btnRemove = new GridData(SWT.LEFT, SWT.CENTER, false, false, 1, 1);
		gd_btnRemove.widthHint = 50;
		btnRemove.setLayoutData(gd_btnRemove);
		btnRemove.setText("Remove");
		if (microservice != null) {
			microserviceTags = microservice.getTags().stream().collect(Collectors.toSet());
		}
		listViewer.setContentProvider(new TagContentProvider());
		listViewer.setInput(microserviceTags);

		btnAdd.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				addToList(microservice, appDescription, comboViewer, combo);

			}
		});
		combo.addTraverseListener(new TraverseListener() {
			public void keyTraversed(TraverseEvent e) {
				if(e.detail == SWT.TRAVERSE_RETURN) {
					addToList(microservice, appDescription, comboViewer, combo);
				}
			}
		});

		btnRemove.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {

				if (list.getSelectionCount() > 0) {

					for (String s : list.getSelection()) {
						microserviceTags.remove(s);
						microservice.setNfrs(microserviceTags.stream().collect(Collectors.toSet()).stream()
								.collect(Collectors.toList()));
					}

					if (appDescription != null) {
						allTags = appDescriptionHelper.getTags();
					} else {
						allTags = new HashSet<String>();
					}
					System.out.println(allTags);
					comboViewer.refresh();
					listViewer.refresh(true);
				}
			}
		});
	}
	/**
	 * @wbp.parser.constructor
	 */
	public TagListViewer(Composite parent, int style, Microservice microservice, AppDescription appDescription) {
		this(parent, SWT.NONE,microservice, appDescription,SWTResourceManager.getColor(SWT.COLOR_WIDGET_BACKGROUND));
	}
	public java.util.List<Resource> getSelection() {

		java.util.List<Resource> selection = listViewer.getStructuredSelection().toList();

		return selection;

	}

	public void addSelection(java.util.List<String> nfrs) {

		HashSet<String> selection = new HashSet<String>(Arrays.asList(list.getSelection()));

		selection.addAll(nfrs);
		list.setSelection(selection.toArray(new String[selection.size()]));

	}

	@Override
	protected void checkSubclass() {
		// Disable the check that prevents subclassing of SWT components
	}

	public void addSelectionListener(SelectionAdapter a) {
		list.addSelectionListener(a);
	}

	private void addToList(Microservice microservice, AppDescription appDescription, ComboViewer comboViewer,
			Combo combo) {
		if (combo.getText().isEmpty())
			return;
		microserviceTags.add(combo.getText());
		microservice.setNfrs(
				microserviceTags.stream().collect(Collectors.toSet()).stream().collect(Collectors.toList()));

		if (appDescription != null) {
			allTags = appDescriptionHelper.getTags();
		} else {
			allTags = new HashSet<String>();
		}
		combo.setText("");
		comboViewer.refresh();
		listViewer.refresh(true);
		combo.setFocus();
	}

}
