/*
* Copyright (c) 2017 Fraunhofer FOKUS.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the
* Eclipse Public License version 2.0 which accompanies
* this distribution, and is available at
* https://opensource.org/licenses/EPL-2.0
*
* Contributors:
*
* Torben Jastrow (Fraunhofer FOKUS) 
* 
* **Initially developed in the context of DECIDE EU project www.DECIDE-h2020.eu */
package eu.DECIDEh2020.architect.patterns.composites;

import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.layout.GridLayout;




import eu.DECIDEh2020.appManager.models.Pattern;
import eu.DECIDEh2020.architect.plugin.utils.CloudPatternHelper;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.wb.swt.SWTResourceManager;

import com.fasterxml.jackson.databind.node.ObjectNode;

import org.eclipse.swt.layout.GridData;

public class PatternDetailComposite extends Composite {

	private Text txtPatternTitle;
	private Text txtSubject;
	private Text txtDescription;
	private Text txtContext;
	private Text txtSolution;

	/**
	 * Create the composite.
	 * @param parent
	 * @param style
	 */

	public PatternDetailComposite(Composite parent, int style) {
		super(parent, style);
		createGUI();
	}

	private void createGUI() {
		setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		setLayout(new GridLayout(2, false));
		
		Label lblPatternTitle = new Label(this, SWT.NONE);
		lblPatternTitle.setLayoutData(new GridData(SWT.LEFT, SWT.TOP, false, false, 1, 1));
		lblPatternTitle.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		lblPatternTitle.setText("Title:");
		
		txtPatternTitle = new Text(this, SWT.BORDER | SWT.READ_ONLY);
		txtPatternTitle.setText("Fundamental Pattern");
		txtPatternTitle.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		
		Label lblSubject = new Label(this, SWT.NONE);
		lblSubject.setLayoutData(new GridData(SWT.LEFT, SWT.TOP, false, false, 1, 1));
		lblSubject.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		lblSubject.setText("Subject:");
		
		txtSubject = new Text(this, SWT.BORDER | SWT.READ_ONLY | SWT.WRAP | SWT.V_SCROLL);
		txtSubject.setText("subject");
		GridData gd_txtSubject = new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1);
		gd_txtSubject.heightHint = 50;
		txtSubject.setLayoutData(gd_txtSubject);
		
		Label lblDescription = new Label(this, SWT.NONE);
		lblDescription.setLayoutData(new GridData(SWT.LEFT, SWT.TOP, false, false, 1, 1));
		lblDescription.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		lblDescription.setText("Description:");
		
		txtDescription = new Text(this, SWT.BORDER | SWT.READ_ONLY | SWT.WRAP | SWT.V_SCROLL | SWT.MULTI);
		txtDescription.setText("description");
		GridData gd_txtDescription = new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1);
		gd_txtDescription.heightHint = 75;
		txtDescription.setLayoutData(gd_txtDescription);
		
		Label lblContext = new Label(this, SWT.NONE);
		lblContext.setLayoutData(new GridData(SWT.LEFT, SWT.TOP, false, false, 1, 1));
		lblContext.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		lblContext.setText("Context:");
		
		txtContext = new Text(this, SWT.BORDER | SWT.READ_ONLY | SWT.WRAP | SWT.V_SCROLL);
		txtContext.setText("context");
		GridData gd_txtContext = new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1);
		gd_txtContext.heightHint = 75;
		txtContext.setLayoutData(gd_txtContext);
		
		Label lblSolution = new Label(this, SWT.NONE);
		lblSolution.setLayoutData(new GridData(SWT.LEFT, SWT.TOP, false, false, 1, 1));
		lblSolution.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		lblSolution.setText("Solution:");
		
		txtSolution = new Text(this, SWT.BORDER | SWT.READ_ONLY | SWT.WRAP | SWT.V_SCROLL | SWT.MULTI);
		txtSolution.setText("solution");
		GridData gd_txtSolution = new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1);
		gd_txtSolution.heightHint = 75;
		txtSolution.setLayoutData(gd_txtSolution);
		updatePattern(null);
	}

	public void updatePattern(Pattern pattern) {

		if (pattern != null) {
			ObjectNode node = CloudPatternHelper.getPattern(pattern.getUriRef());
			
			txtPatternTitle.setText(pattern.getTitle() !=null ? pattern.getTitle() : "");
			txtSubject.setText(node.path("subject").asText(""));
			txtDescription.setText(node.path("description").asText(""));
			txtContext.setText(node.path("context").asText(""));
			txtSolution.setText(node.path("solution").asText(""));
		} else {
			txtPatternTitle.setText("");
			txtSubject.setText("");
			txtDescription.setText("");
			txtContext.setText("");
			txtSolution.setText("");
		}
		
		redraw();
	}

	@Override
	protected void checkSubclass() {
		// Disable the check that prevents subclassing of SWT components
	}

}