/*
 * Copyright (c) 2017 Fraunhofer FOKUS.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the
 * GNU AGPL v3 which accompanies
 * this distribution, and is available at
 * http://www.gnu.org/licenses/agpl.txt
 *
 * Contributors:
 * Simon Dutkowski Fraunhofer FOKUS
 *
 * Initially developed in the context of DECIDE EU project www.DECIDE-h2020.eu
 */
package eu.DECIDEh2020.architect.cloudpatterns;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.ArrayNode;
import com.fasterxml.jackson.databind.node.ObjectNode;
import org.apache.jena.rdf.model.Resource;
import org.apache.jena.sparql.vocabulary.FOAF;
import org.apache.jena.vocabulary.DCTerms;

public class PatternConverter {

    public static Resource patternToResource(ObjectNode nfr) {
        return null;
    }

    public static ObjectNode patternToJson(Resource pattern) {
        ObjectNode obj = new ObjectMapper().createObjectNode();
        obj.put("uriRef", pattern.getURI());
        obj.put("title", pattern.getRequiredProperty(DCTerms.title).getString());
        if (pattern.hasProperty(DCTerms.type)) {
            ArrayNode array = obj.putArray("categories");
            pattern.listProperties(DCTerms.type).forEachRemaining(stmt -> array.add(stmt.getResource().getURI()));
        }
        if (pattern.hasProperty(DP.icon)) {
            obj.put("icon", pattern.getPropertyResourceValue(DP.icon).getLocalName());
        }
        if (pattern.hasProperty(DCTerms.subject)) {
            obj.put("subject", pattern.getProperty(DCTerms.subject).getString());
        }
        if (pattern.hasProperty(DCTerms.description)) {
            obj.put("description", pattern.getProperty(DCTerms.description).getString());
        }
        if (pattern.hasProperty(DP.context)) {
            obj.put("context", pattern.getProperty(DP.context).getString());
        }
        if (pattern.hasProperty(DP.solution)) {
            obj.put("solution", pattern.getProperty(DP.solution).getString());
        }
        if (pattern.hasProperty(DCTerms.license)) {
            obj.put("license", pattern.getPropertyResourceValue(DCTerms.license).getLocalName());
        }
        if (pattern.hasProperty(FOAF.page)) {
            ObjectNode page = obj.putObject("page");
            Resource p = pattern.getPropertyResourceValue(FOAF.page);
            page.put("topic", p.getProperty(FOAF.topic).getString());
            page.put("primaryTopic", p.getPropertyResourceValue(FOAF.primaryTopic).getURI());
        }
        if (pattern.hasProperty(DCTerms.relation)) {
            ArrayNode array = obj.putArray("relations");
            pattern.listProperties(DCTerms.relation).forEachRemaining(stmt -> array.add(stmt.getResource().getURI()));
        }
        if (pattern.hasProperty(DP.hasImpactOn)) {
            ArrayNode array = obj.putArray("impacts");
            pattern.listProperties(DP.hasImpactOn).forEachRemaining(stmt -> array.add(stmt.getResource().getURI()));
        }
        if (pattern.hasProperty(DP.hasPositiveImpactOn)) {
            ArrayNode array = obj.putArray("positiveImpacts");
            pattern.listProperties(DP.hasPositiveImpactOn).forEachRemaining(stmt -> array.add(stmt.getResource().getURI()));
        }
        if (pattern.hasProperty(DP.hasNegativeImpactOn)) {
            ArrayNode array = obj.putArray("negativeImpacts");
            pattern.listProperties(DP.hasNegativeImpactOn).forEachRemaining(stmt -> array.add(stmt.getResource().getURI()));
        }
        return obj;
    }

    public static ObjectNode patternToJsonReduced(Resource pattern) {
        ObjectNode obj = new ObjectMapper().createObjectNode();
        obj.put("uriRef", pattern.getURI());
        obj.put("title", pattern.getRequiredProperty(DCTerms.title).getString());
        if (pattern.hasProperty(DCTerms.type)) {
            ArrayNode array = obj.putArray("categories");
            pattern.listProperties(DCTerms.type).forEachRemaining(stmt -> array.add(stmt.getResource().getURI()));
        }
        if (pattern.hasProperty(DP.hasPositiveImpactOn)) {
            ArrayNode array = obj.putArray("positiveImpacts");
            pattern.listProperties(DP.hasPositiveImpactOn).forEachRemaining(stmt -> array.add(stmt.getResource().getURI()));
        }
        return obj;
    }

}
